from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from nose.tools import assert_false, assert_true, assert_equal
from iminuit.util import (fitarg_rename,
                          true_param,
                          param_name,
                          extract_iv,
                          extract_limit,
                          extract_error,
                          extract_fix,
                          remove_var,
                          arguments_from_docstring,
                          )


def test_fitarg_rename():
    fitarg = {'x': 1, 'limit_x': (2, 3), 'fix_x': True, 'error_x': 10}
    ren = lambda x: 'z_' + x
    newfa = fitarg_rename(fitarg, ren)
    assert_true('z_x' in newfa)
    assert_true('limit_z_x' in newfa)
    assert_true('error_z_x' in newfa)
    assert_true('fix_z_x' in newfa)
    assert_equal(len(newfa), 4)


def test_fitarg_rename_strprefix():
    fitarg = {'x': 1, 'limit_x': (2, 3), 'fix_x': True, 'error_x': 10}
    newfa = fitarg_rename(fitarg, 'z')
    assert_true('z_x' in newfa)
    assert_true('limit_z_x' in newfa)
    assert_true('error_z_x' in newfa)
    assert_true('fix_z_x' in newfa)
    assert_equal(len(newfa), 4)


def test_true_param():
    assert_true(true_param('N'))
    assert_false(true_param('limit_N'))
    assert_false(true_param('error_N'))
    assert_false(true_param('fix_N'))


def test_param_name():
    assert_equal(param_name('N'), 'N')
    assert_equal(param_name('limit_N'), 'N')
    assert_equal(param_name('error_N'), 'N')
    assert_equal(param_name('fix_N'), 'N')


def test_extract_iv():
    d = dict(k=1., limit_k=1., error_k=1., fix_k=1.)
    ret = extract_iv(d)
    assert_true('k' in ret)
    assert_false('limit_k' in ret)
    assert_false('error_k' in ret)
    assert_false('fix_k' in ret)


def test_extract_limit():
    d = dict(k=1., limit_k=1., error_k=1., fix_k=1.)
    ret = extract_limit(d)
    assert_false('k' in ret)
    assert_true('limit_k' in ret)
    assert_false('error_k' in ret)
    assert_false('fix_k' in ret)


def test_extract_error():
    d = dict(k=1., limit_k=1., error_k=1., fix_k=1.)
    ret = extract_error(d)
    assert_false('k' in ret)
    assert_false('limit_k' in ret)
    assert_true('error_k' in ret)
    assert_false('fix_k' in ret)


def test_extract_fix():
    d = dict(k=1., limit_k=1., error_k=1., fix_k=1.)
    ret = extract_fix(d)
    assert_false('k' in ret)
    assert_false('limit_k' in ret)
    assert_false('error_k' in ret)
    assert_true('fix_k' in ret)


def test_remove_var():
    dk = dict(k=1, limit_k=1, error_k=1, fix_k=1)
    dl = dict(l=1, limit_l=1, error_l=1, fix_l=1)
    dm = dict(m=1, limit_m=1, error_m=1, fix_m=1)
    dn = dict(n=1, limit_n=1, error_n=1, fix_n=1)
    d = {}
    d.update(dk)
    d.update(dl)
    d.update(dm)
    d.update(dn)

    ret = remove_var(d, ['k', 'm'])
    for k in dk: assert_false(k in ret)
    for k in dl: assert_true(k in ret)
    for k in dm: assert_false(k in ret)
    for k in dn: assert_true(k in ret)


def test_arguments_from_docstring():
    s = 'f(x, y, z)'
    a = arguments_from_docstring(s)
    assert_equal(a, ['x', 'y', 'z'])
    # this is a hard one
    s = 'Minuit.migrad( int ncall_me =10000, [resume=True, int nsplit=1])'
    a = arguments_from_docstring(s)
    assert_equal(a, ['ncall_me', 'resume', 'nsplit'])



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

__all__ = ['ConsoleFrontend']


class ConsoleFrontend:
    """Console frontend for Minuit.

    This class prints stuff directly via print.
    """

    @staticmethod
    def print_fmin(sfmin, tolerance=None, ncalls=0):
        """display function minimum information
        for FunctionMinimumStruct *sfmin*.
        It contains various migrad status.
        """
        fmin = sfmin
        goaledm = 0.0001 * tolerance * fmin.up if tolerance is not None else ''
        # despite what the doc said the code is actually 1e-4
        # http://wwwasdoc.web.cern.ch/wwwasdoc/hbook_html3/node125.html
        flatlocal = dict(list(locals().items()) + list(fmin.__dict__.items()))
        info1 = 'fval = %(fval)r | total call = %(ncalls)r | ncalls = %(nfcn)r\n' % \
                flatlocal
        info2 = 'edm = %(edm)r (Goal: %(goaledm)r) | up = %(up)r\n' % flatlocal
        header1 = '|' + (' %14s |' * 5) % (
            'Valid',
            'Valid Param',
            'Accurate Covar',
            'Posdef',
            'Made Posdef') + '\n'
        hline = '-' * len(header1) + '\n'
        status1 = '|' + (' %14r |' * 5) % (
            fmin.is_valid,
            fmin.has_valid_parameters,
            fmin.has_accurate_covar,
            fmin.has_posdef_covar,
            fmin.has_made_posdef_covar) + '\n'
        header2 = '|' + (' %14s |' * 5) % (
            'Hesse Fail',
            'Has Cov',
            'Above EDM',
            '',
            'Reach calllim') + '\n'
        status2 = '|' + (' %14r |' * 5) % (
            fmin.hesse_failed,
            fmin.has_covariance,
            fmin.is_above_max_edm,
            '',
            fmin.has_reached_call_limit) + '\n'

        print(hline + info1 + info2 +
              hline + header1 + hline + status1 +
              hline + header2 + hline + status2 +
              hline)

    @staticmethod
    def print_merror(vname, smerr):
        """print minos error for varname"""
        stat = 'VALID' if smerr.is_valid else 'PROBLEM'

        summary = 'Minos Status for %s: %s\n' % \
                  (vname, stat)

        error = '| {0:^15s} | {1: >12g} | {2: >12g} |\n'.format(
            'Error',
            smerr.
                lower,
            smerr.upper)
        valid = '| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
            'Valid',
            str(smerr.lower_valid),
            str(smerr.upper_valid))
        at_limit = '| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
            'At Limit',
            str(smerr.at_lower_limit),
            str(smerr.at_upper_limit))
        max_fcn = '| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
            'Max FCN',
            str(smerr.at_lower_max_fcn),
            str(smerr.at_upper_max_fcn))
        new_min = '| {0:^15s} | {1:^12s} | {2:^12s} |\n'.format(
            'New Min',
            str(smerr.lower_new_min),
            str(smerr.upper_new_min))
        hline = '-' * len(error) + '\n'
        print(hline +
              summary +
              hline +
              error +
              valid +
              at_limit +
              max_fcn +
              new_min +
              hline)

    @staticmethod
    def print_param(mps, merr=None, float_format=None):
        """Print parameter states

        Arguments:

            *mps*: list of MinuitParameter struct

            *merr*: dictionary of vname->minos error struct

            *float_format*: ignored
        """
        merr = {} if merr is None else merr
        vnames = [mp.name for mp in mps]
        maxlength = max([len(x) for x in vnames])
        maxlength = max(5, maxlength)

        header = (('| {0:^4s} | {1:^%ds} | {2:^8s} | {3:^8s} | {4:^8s} |'
                   ' {5:^8s} | {6:^8s} | {7:^8s} | {8:^8s} |\n') % maxlength).format(
            '', 'Name', 'Value', 'Para Err',
            "Err-", "Err+", "Limit-", "Limit+", " ")
        hline = '-' * len(header) + '\n'
        linefmt = ('| {0:>4d} | {1:>%ds} = {2:<8s} | {3:<8s} | {4:<8s} |'
                   ' {5:<8s} | {6:<8s} | {7:<8s} | {8:^8s} |\n') % maxlength
        nfmt = '{0:< 8.4G}'
        nformat = nfmt.format
        blank = ' ' * 8

        ret = hline + header + hline
        for i, (v, mp) in enumerate(zip(vnames, mps)):
            tmp = [i, v]

            tmp.append(nfmt.format(mp.value))
            tmp.append(nfmt.format(mp.error))

            tmp.append(nformat(merr[v].lower) if v in merr else blank)
            tmp.append(nformat(merr[v].upper) if v in merr else blank)

            tmp.append(nformat(mp.lower_limit) if mp.has_upper_limit else blank)
            tmp.append(nformat(mp.upper_limit) if mp.has_lower_limit else blank)

            tmp.append(
                'FIXED' if mp.is_fixed else 'CONST' if mp.is_const else '')

            line = linefmt.format(*tmp)
            ret += line
        ret += hline
        print(ret)

    @staticmethod
    def print_banner(cmd):
        """show banner of command"""
        ret = '*' * 50 + '\n'
        ret += '*{:^48}*'.format(cmd) + '\n'
        ret += '*' * 50 + '\n'
        print(ret)

    @staticmethod
    def print_matrix(vnames, matrix):
        """TODO: properly implement this"""
        print(vnames)
        print(matrix)
        maxlen = max(len(v) for v in vnames)
        narg = len(matrix)
        vfmt = '%%%ds' % maxlen
        vblank = ' ' * maxlen
        fmt = '%3.2f '  # 4char
        dfmt = '%4d '
        tmp = ''
        header = vblank + ' ' * 4 + '  | ' + (dfmt * narg) % tuple(range(narg)) + '\n'
        blank_line = '-' * len(header) + '\n'
        tmp += header + blank_line
        for i, (v, row) in enumerate(zip(vnames, matrix)):
            fmt = '%3.2f ' * narg
            head = (vfmt + ' %4d | ') % (v, i)
            content = (fmt) % tuple(row) + '\n'
            tmp += head + content
        tmp += blank_line
        print(tmp)

    def print_hline(self):
        print('*' * 70)



"""IMinuit is a nice Python interface to MINUIT.

Basic usage example::

    from iminuit import Minuit
    def f(x, y, z):
        return (x - 2) ** 2 + (y - 3) ** 2 + (z - 4) ** 2
    m = Minuit(f)
    m.migrad()
    print(m.values)  # {'x': 2,'y': 3,'z': 4}
    print(m.errors)  # {'x': 1,'y': 1,'z': 1}

Further information:

* Code: https://github.com/iminuit/iminuit
* Docs: http://iminuit.readthedocs.org/
"""

__all__ = [
    'Minuit',
    'describe',
    'Struct',
    'InitialParamWarning',
    '__version__'
]

from iminuit._libiminuit import *
from iminuit.util import describe, Struct
from iminuit.iminuit_warnings import *
from iminuit.info import __version__

# Define a test function that runs the `iminuit` tests.
# This seems to work OK
# http://docs.scipy.org/doc/numpy/reference/generated/numpy.testing.Tester.test.html
# try:
#     from numpy.testing import Tester
#     test = Tester().test
# except:
#     pass
# An alternative would be to use `nose` directly to avoid the `numpy` depencency
# `scikit-image` is a nice example we could maybe copy & paste 
# https://github.com/scikit-image/scikit-image/blob/master/skimage/__init__.py



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import warnings
from math import sqrt
from unittest import TestCase
from nose.tools import (raises,
                        assert_equal,
                        assert_true,
                        assert_false,
                        assert_almost_equal,
                        )
from iminuit import Minuit

try:
    from numpy.testing import assert_array_almost_equal
except ImportError:
    def assert_array_almost_equal(actual, expected, decimal=6):
        """
        Helper function to test if all elements of a list of lists
        are almost equal.
        A replacement for numpy.testing.assert_array_almost_equal,
        if it is not installed
        """
        for row in range(len(actual)):
            for col in range(len(actual[0])):
                assert_almost_equal(actual[row][col], expected[row][col], places=decimal)


class Func_Code:
    def __init__(self, varname):
        self.co_varnames = varname
        self.co_argcount = len(varname)


class Func1:
    def __init__(self):
        pass

    def __call__(self, x, y):
        return (x - 2.) ** 2 + (y - 5.) ** 2 + 10


class Func2:
    def __init__(self):
        self.func_code = Func_Code(['x', 'y'])

    def __call__(self, *arg):
        return (arg[0] - 2.) ** 2 + (arg[1] - 5.) ** 2 + 10


def func3(x, y):
    return 0.2 * (x - 2.) ** 2 + (y - 5.) ** 2 + 10


def func4(x, y, z):
    return 0.2 * (x - 2.) ** 2 + 0.1 * (y - 5.) ** 2 + 0.25 * (z - 7.) ** 2 + 10


def func5(x, long_variable_name_really_long_why_does_it_has_to_be_this_long, z):
    return (x ** 2) + (z ** 2) + long_variable_name_really_long_why_does_it_has_to_be_this_long ** 2


def func6(x, m, s, A):
    return A / ((x - m) ** 2 + s ** 2)


data_y = [0.552, 0.735, 0.846, 0.875, 1.059, 1.675, 1.622, 2.928,
          3.372, 2.377, 4.307, 2.784, 3.328, 2.143, 1.402, 1.44,
          1.313, 1.682, 0.886, 0.0, 0.266, 0.3]
data_x = list(range(len(data_y)))


def chi2(m, s, A):
    """Chi2 fitting routine"""
    return sum(((func6(x, m, s, A) - y) ** 2 for x, y in zip(data_x, data_y)))


def functesthelper(f):
    m = Minuit(f, print_level=0, pedantic=False)
    m.migrad()
    val = m.values
    assert_almost_equal(val['x'], 2.)
    assert_almost_equal(val['y'], 5.)
    assert_almost_equal(m.fval, 10.)
    assert m.matrix_accurate()
    assert m.migrad_ok()
    return m


def test_f1():
    functesthelper(Func1())


def test_f2():
    functesthelper(Func2())


def test_f3():
    functesthelper(func3)


@raises(RuntimeError)
def test_typo():
    Minuit(func4, printlevel=0)
    # self.assertRaises(RuntimeError,Minuit,func4,printlevel=0)


def test_non_invertible():
    # making sure it doesn't crash
    def f(x, y):
        return (x * y) ** 2

    m = Minuit(f, pedantic=False, print_level=0)
    m.migrad()
    with warnings.catch_warnings():
        warnings.simplefilter('error')
        try:
            m.hesse()
            raise RuntimeError('Hesse did not raise a warning')
        except Warning:
            pass
    try:
        m.matrix()
        raise RuntimeError('Matrix did not raise an error')  # shouldn't reach here
    except RuntimeError as e:
        pass


def test_fix_param():
    m = Minuit(func4, print_level=0, pedantic=False)
    m.migrad()
    m.minos()
    val = m.values
    assert_almost_equal(val['x'], 2.)
    assert_almost_equal(val['y'], 5.)
    assert_almost_equal(val['z'], 7.)
    err = m.errors  # second derivative
    m.print_all_minos()
    # now fix z = 10
    m = Minuit(func4, print_level=-1, y=10., fix_y=True, pedantic=False)
    m.migrad()
    val = m.values
    assert_almost_equal(val['x'], 2.)
    assert_almost_equal(val['y'], 10.)
    assert_almost_equal(val['z'], 7.)
    assert_almost_equal(m.fval, 10. + 2.5)
    free_param = m.list_of_vary_param()
    fix_param = m.list_of_fixed_param()
    assert_true('x' in free_param)
    assert_false('x' in fix_param)
    assert_true('y' in fix_param)
    assert_false('y' in free_param)
    assert_false('z' in fix_param)


def test_fitarg_oneside():
    m = Minuit(func4, print_level=-1, y=10., fix_y=True, limit_x=(None, 20.),
               pedantic=False)
    fitarg = m.fitarg
    assert_almost_equal(fitarg['y'], 10.)
    assert_true(fitarg['fix_y'])
    assert_equal(fitarg['limit_x'], (None, 20))
    m.migrad()

    fitarg = m.fitarg

    assert_almost_equal(fitarg['y'], 10.)
    assert_almost_equal(fitarg['x'], 2., places=2)
    assert_almost_equal(fitarg['z'], 7., places=2)

    assert_true('error_y' in fitarg)
    assert_true('error_x' in fitarg)
    assert_true('error_z' in fitarg)

    assert_true(fitarg['fix_y'])
    assert_equal(fitarg['limit_x'], (None, 20))


def test_fitarg():
    m = Minuit(func4, print_level=-1, y=10., fix_y=True, limit_x=(0, 20.),
               pedantic=False)
    fitarg = m.fitarg
    assert_almost_equal(fitarg['y'], 10.)
    assert_true(fitarg['fix_y'])
    assert_equal(fitarg['limit_x'], (0, 20))
    m.migrad()

    fitarg = m.fitarg

    assert_almost_equal(fitarg['y'], 10.)
    assert_almost_equal(fitarg['x'], 2., places=2)
    assert_almost_equal(fitarg['z'], 7., places=2)

    assert_true('error_y' in fitarg)
    assert_true('error_x' in fitarg)
    assert_true('error_z' in fitarg)

    assert_true(fitarg['fix_y'])
    assert_equal(fitarg['limit_x'], (0, 20))


def test_minos_all():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.migrad()
    m.minos()
    assert_almost_equal(m.merrors[('x', -1.0)], -sqrt(5))
    assert_almost_equal(m.merrors[('x', 1.0)], sqrt(5))
    assert_almost_equal(m.merrors[('y', 1.0)], 1.)


def test_minos_single():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.migrad()
    m.minos('x')
    assert_almost_equal(m.merrors[('x', -1.0)], -sqrt(5))
    assert_almost_equal(m.merrors[('x', 1.0)], sqrt(5))


@raises(RuntimeWarning)
def test_minos_single_fixed_raising():
    m = Minuit(func3, pedantic=False, print_level=0, fix_x=True)
    m.migrad()

    with warnings.catch_warnings():
        warnings.simplefilter('error')
        ret = m.minos('x')


def test_minos_single_fixed_result():
    m = Minuit(func3, pedantic=False, print_level=0, fix_x=True)
    m.migrad()
    with warnings.catch_warnings():
        warnings.simplefilter('ignore')
        ret = m.minos('x')
    assert_equal(ret, None)


@raises(RuntimeError)
def test_minos_single_no_migrad():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.minos('x')


@raises(RuntimeError)
def test_minos_single_nonsense_variable():
    m = Minuit(func3, pedantic=False, print_level=0)
    m.migrad()
    m.minos('nonsense')


def test_fixing_long_variable_name():
    m = Minuit(func5, pedantic=False, print_level=0,
               fix_long_variable_name_really_long_why_does_it_has_to_be_this_long=True,
               long_variable_name_really_long_why_does_it_has_to_be_this_long=0)
    m.migrad()


def test_initial_value():
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    assert_almost_equal(m.args[0], 1.)
    assert_almost_equal(m.args[1], 2.)
    assert_almost_equal(m.values['x'], 1.)
    assert_almost_equal(m.values['y'], 2.)
    assert_almost_equal(m.errors['x'], 3.)


def test_mncontour():
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    xminos, yminos, ctr = m.mncontour('x', 'y', numpoints=30)
    xminos_t = m.minos('x')['x']
    yminos_t = m.minos('y')['y']
    assert_almost_equal(xminos.upper, xminos_t.upper)
    assert_almost_equal(xminos.lower, xminos_t.lower)
    assert_almost_equal(yminos.upper, yminos_t.upper)
    assert_almost_equal(yminos.lower, yminos_t.lower)
    assert_equal(len(ctr), 30)
    assert_equal(len(ctr[0]), 2)


def test_mncontour_sigma():
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    xminos, yminos, ctr = m.mncontour('x', 'y', numpoints=30, sigma=2.0)
    xminos_t = m.minos('x', sigma=2.0)['x']
    yminos_t = m.minos('y', sigma=2.0)['y']
    assert_almost_equal(xminos.upper, xminos_t.upper)
    assert_almost_equal(xminos.lower, xminos_t.lower)
    assert_almost_equal(yminos.upper, yminos_t.upper)
    assert_almost_equal(yminos.lower, yminos_t.lower)
    assert_equal(len(ctr), 30)
    assert_equal(len(ctr[0]), 2)


def test_contour():
    # FIXME: check the result
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    m.contour('x', 'y')


def test_profile():
    # FIXME: check the result
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    m.profile('y')


def test_mnprofile():
    # FIXME: check the result
    m = Minuit(func3, pedantic=False, x=1., y=2., error_x=3., print_level=0)
    m.migrad()
    m.mnprofile('y')


@raises(RuntimeError)
def test_printfmin_uninitialized():
    # issue 85
    def f(x): return 2 + 3 * x

    fitter = Minuit(f, pedantic=False)
    fitter.print_fmin()


@raises(ValueError)
def test_reverse_limit():
    # issue 94
    def f(x, y, z):
        return (x - 2) ** 2 + (y - 3) ** 2 + (z - 4) ** 2

    m = Minuit(f, limit_x=(3., 2.), pedantic=False)
    m.migrad()


class TestMatrix(TestCase):
    def setUp(self):
        self.m = Minuit(func3, print_level=0, pedantic=False)
        self.m.migrad()

    def test_matrix(self):
        actual = self.m.np_matrix()
        expected = [[5., 0.], [0., 1.]]
        assert_array_almost_equal(actual, expected)

    def test_np_matrix(self):
        import numpy as np
        actual = self.m.np_matrix()
        expected = [[5., 0.], [0., 1.]]
        assert_array_almost_equal(actual, expected)
        assert isinstance(actual, np.ndarray)

    def test_matrix_correlation(self):
        actual = self.m.matrix(correlation=True)
        expected = [[1., 0.], [0., 1.]]
        assert_array_almost_equal(actual, expected)

    def test_np_matrix_correlation(self):
        import numpy as np
        actual = self.m.np_matrix(correlation=True)
        expected = [[1., 0.], [0., 1.]]
        assert_array_almost_equal(actual, expected)
        assert isinstance(actual, np.ndarray)


def test_chi2_fit():
    """Fit a curve to data."""
    m = Minuit(chi2, s=2., error_A=0.1, errordef=0.01,
               print_level=0, pedantic=False)
    m.migrad()
    output = [round(10 * m.values['A']), round(100 * m.values['s']),
              round(100 * m.values['m'])]
    expected = [round(10 * 64.375993), round(100 * 4.267970),
                round(100 * 9.839172)]
    assert_array_almost_equal(output, expected)


def test_oneside():
    m_limit = Minuit(func3, limit_x=(None, 9), pedantic=False, print_level=0)
    m_nolimit = Minuit(func3, pedantic=False, print_level=0)
    # Solution: x=2., y=5.
    m_limit.tol = 1e-4
    m_nolimit.tol = 1e-4
    m_limit.migrad()
    m_nolimit.migrad()
    assert_array_almost_equal(list(m_limit.values.values()),
                              list(m_nolimit.values.values()), decimal=4)


def test_oneside_outside():
    m = Minuit(func3, limit_x=(None, 1), pedantic=False, print_level=0)
    m.migrad()
    assert_almost_equal(m.values['x'], 1)



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from nose.tools import assert_equal
from iminuit.latex import LatexTable


def test_table():
    ltt = LatexTable(data=[['alpha', 10.123], ['alpha_s', 30]])
    assert_equal(str(ltt),
                 '\\begin{tabular}{|c|c|}\n\\hline\n$\\alpha$ &     10.123\\\\\n\\hline\n$\\alpha_{s}$ & 30\\\\\n\\hline\n\\end{tabular}'
                 )


def test_table_color():
    ltt = LatexTable(data=[['alpha', 10.123], ['alpha_s', 30]])
    ltt.set_cell_color(0, 0, (111, 123, 95))
    assert_equal(str(ltt),
                 '%\\usepackage[table]{xcolor} % include this for color\n%\\usepackage{rotating} % include this for rotate header\n%\\documentclass[xcolor=table]{beamer} % for beamer\n\\begin{tabular}{|c|c|}\n\\hline\n\\cellcolor[RGB]{111,123,95} $\\alpha$ &     10.123\\\\\n\\hline\n$\\alpha_{s}$ & 30\\\\\n\\hline\n\\end{tabular}'
                 )


def test_latexmap():
    ltt = LatexTable(data=[['alpha', 10.123, 20], ['alpha_s', 30, 40]],
                     latex_map={'alpha': 'beta'})
    assert_equal(
        ltt._format('alpha'),
        'beta'
    )


def test_smartlatex():
    ltt = LatexTable(data=[['alpha', 10.123, 20], ['alpha_s', 30, 40]])
    assert_equal(
        ltt._convert_smart_latex('alpha'),
        r'$\alpha$'
    )
    assert_equal(
        ltt._convert_smart_latex('alpha_beta'),
        r'$\alpha_{\beta}$'
    )
    assert_equal(
        ltt._convert_smart_latex('a_b'),
        r'$a_{b}$'
    )
    assert_equal(
        ltt._convert_smart_latex('a_b_c'),
        r'a $b_{c}$'
    )
    assert_equal(
        ltt._convert_smart_latex('a'),
        r'a'
    )
    assert_equal(
        ltt._convert_smart_latex('a_alpha_beta'),
        r'a $\alpha_{\beta}$'
    )


def test_format():
    ltt = LatexTable(data=[['alpha', 10.123, 20], ['alpha_s', 30, 40]],
                     smart_latex=False)
    assert_equal(ltt._format('a_b'), r'a\_b')
    ltt2 = LatexTable(data=[['alpha', 10.123, 20], ['alpha_s', 30, 40]],
                      smart_latex=False, escape_under_score=False)
    assert_equal(ltt2._format('a_b'), r'a_b')
